"""
Configuration file for the Python Atmospheric data Community Toolkit (ACT)
The values for a number of ACT parameters and the default metadata created
when reading files, correcting fields, etc. is controlled by this single
Python configuration file.

Examples:
---------
from act.config import DEFAULT_DATASTREAM_NAME

"""

DEFAULT_DATASTREAM_NAME = 'act_datastream'
